/* $Id: globus_i_replica_catalog.h,v 1.9 2001/03/22 20:24:49 meder Exp $ */

#ifndef GLOBUS_DONT_DOCUMENT_INTERNAL

/**
 * @file
 *
 * Globus Replica Catalog Internal C API Header File
 */

#ifndef _GLOBUS_I_REPLICA_CATALOG_H
#define _GLOBUS_I_REPLICA_CATALOG_H

#ifdef __cplusplus
#define EXTERN_C_BEGIN extern "C" {
#define EXTERN_C_END }
#else
#define EXTERN_C_BEGIN
#define EXTERN_C_END
#endif


EXTERN_C_BEGIN

/**
 * Compare handle attribute structures.
 */
int
globus_i_replica_catalog_collection_handleattr_compare(
    globus_replica_catalog_collection_handleattr_t *    attrs1,
    globus_replica_catalog_collection_handleattr_t *    attrs2);

globus_result_t
globus_i_replica_catalog_location_get_url(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char *                                          filename,
    char **                                         url);
    
/* Rollback and Restart Information */

/** Default objectclass name of a rollback information entry. */
#define GLOBUS_REPLICA_CATALOG_ROLLBACK_INFO_OBJECTCLASS "GlobusReplicaLogicalRollbackInfo"

/** RDN attribute name of a rollback information entry. */
#define GLOBUS_REPLICA_CATALOG_ROLLBACK_INFO_RDN_TYPE "ri"

/**
 * Construct the contact string for the rollback and restart information
 * for a logical file in a location.
 *
 * Resulting contact string must be freed by the caller with
 * globus_libc_free().
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  location name of the target
 * @param logicalfile_name
 * 	  logical file name of the target
 * @param contact_string
 * 	  contact string @b returned
 */
globus_result_t
globus_i_replica_catalog_rollback_info_construct_contact(
    globus_replica_catalog_collection_handle_t * handle,
    char * location_name,
    char * logicalfile_name,
    char ** contact_string );

/**
 * Parse the rollback and restart contact string into an LDAP URL,
 * location name, and logical file name.
 *
 * Resulting URL and names must be freed by the caller with
 * globus_libc_free().
 *
 * @param contact_string
 * 	  contact string
 * @param url
 * 	  LDAP URL of the collection DN @b returned
 * @param location_name
 * 	  location name @b returned
 * @param logicalfile_name
 * 	  logical file name @b returned
 */
globus_result_t
globus_i_replica_catalog_rollback_info_parse_contact(
    char * contact_string,
    char ** url,
    char ** location_name,
    char ** logicalfile_name );

/**
 * Create rollback and restart information for a logical file in a
 * location.
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  location name of the target
 * @param logicalfile_name
 * 	  logical file name of the target
 * @param operation
 * 	  name of the operation
 * @param state
 * 	  start state of the operation
 * @param session_id
 * 	  session id associated with an operation, may be GLOBUS_NULL
 * @param source_url
 * 	  URL of the source of the operation, may be GLOBUS_NULL
 */
globus_result_t
globus_i_replica_catalog_rollback_info_create(
    globus_replica_catalog_collection_handle_t * handle,
    char * location_name,
    char * logicalfile_name,
    char * operation,
    char * state,
    char * session_id,
    char * source_url );

/**
 * Update rollback and restart information for a logical file in a
 * location with current byte transfer information.
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  location name of the target
 * @param logicalfile_name
 * 	  logical file name of the target
 * @param state
 * 	  completed operation state, if GLOBUS_NULL it will
 * 	  retain its previous value
 * @param ranges
 * 	  comma-separated list of completed ranges, if GLOBUS_NULL it
 * 	  will retain its previous value
 */
globus_result_t
globus_i_replica_catalog_rollback_info_update(
    globus_replica_catalog_collection_handle_t * handle,
    char * location_name,
    char * logicalfile_name,
    char * state,
    char * ranges );

/**
 * Get rollback and restart information for a logical file in a
 * location.
 *
 * Resulting information must be freed by the caller with
 * globus_libc_free().
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  location name of the target
 * @param logicalfile_name
 * 	  logical file name of the target
 * @param operation
 * 	  name of the operation @b returned
 * @param state
 * 	  start state of the operation @b returned
 * @param session_id
 * 	  session id associated with an operation @ returned
 * @param source_url
 * 	  URL of the source of the operation @b returned
 * @param ranges
 * 	  comma-separated list of completed ranges @b returned
 */
globus_result_t
globus_i_replica_catalog_rollback_info_get(
    globus_replica_catalog_collection_handle_t * handle,
    char * location_name,
    char * logicalfile_name,
    char ** operation,
    char ** state,
    char ** session_id,
    char ** source_url,
    char ** ranges );
    
/**
 * Delete rollback and restart information for a logical file in a
 * location.
 *
 * @param handle
 * 	  handle to the collection
 * @param location_name
 * 	  location name of the target
 * @param logicalfile_name
 * 	  logical file name of the target
 */
globus_result_t
globus_i_replica_catalog_rollback_info_delete(
    globus_replica_catalog_collection_handle_t * handle,
    char * location_name,
    char * logicalfile_name );


globus_result_t
globus_i_replica_catalog_location_delete_filenames_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char **                                         filenames);

globus_result_t
globus_i_replica_catalog_rollback_info_update_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char *                                          logicalfile_name,
    char *                                          state,
    char *                                          ranges);

globus_result_t
globus_i_replica_catalog_rollback_info_delete_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char *                                          logicalfile_name);

globus_result_t
globus_i_replica_catalog_location_add_filenames_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char *                                          location_name,
    char **                                         filenames,
    globus_bool_t                                   add);


globus_result_t
globus_i_replica_catalog_collection_add_filenames_wrapper(
    globus_replica_catalog_collection_handle_t *    handle,
    char **                                         filenames,
    globus_bool_t                                   add);


EXTERN_C_END

#endif /* _GLOBUS_I_REPLICA_CATALOG_H */

#endif /* GLOBUS_DONT_DOCUMENT_INTERNAL */

